aboutsummaryrefslogtreecommitdiff
path: root/libbuild2/parser.cxx
diff options
context:
space:
mode:
authorBoris Kolpackov <boris@codesynthesis.com>2024-01-15 11:12:14 +0200
committerBoris Kolpackov <boris@codesynthesis.com>2024-01-15 11:12:14 +0200
commit56cc47e7e5dd3aa15e1023db5525bdc6a5486c1f (patch)
treef2df51d7719f32306787ace9a73223d4a2303f47 /libbuild2/parser.cxx
parentc5187f8198fa8459b6219085f1f8d94bc280e9c5 (diff)
Add no_default_target attribute for source, buildfile import directives
This attribute can be used to disable the default target semantics for the sources/imported buildfile.
Diffstat (limited to 'libbuild2/parser.cxx')
-rw-r--r--libbuild2/parser.cxx68
1 files changed, 56 insertions, 12 deletions
diff --git a/libbuild2/parser.cxx b/libbuild2/parser.cxx
index a404115..0fcf977 100644
--- a/libbuild2/parser.cxx
+++ b/libbuild2/parser.cxx
@@ -3181,7 +3181,7 @@ namespace build2
source_buildfile (istream& is,
const path_name& in,
const location& loc,
- bool deft)
+ optional<bool> deft)
{
tracer trace ("parser::source_buildfile", &path_);
@@ -3199,11 +3199,11 @@ namespace build2
lexer_ = &l;
target* odt;
- if (deft)
- {
+ if (!deft || *deft)
odt = default_target_;
+
+ if (deft && *deft)
default_target_ = nullptr;
- }
token t;
type tt;
@@ -3213,13 +3213,14 @@ namespace build2
if (tt != type::eos)
fail (t) << "unexpected " << t;
- if (deft)
+ if (deft && *deft)
{
if (stage_ != stage::boot && stage_ != stage::root)
process_default_target (t, bf);
+ }
+ if (!deft || *deft)
default_target_ = odt;
- }
lexer_ = ol;
path_ = op;
@@ -3230,11 +3231,35 @@ namespace build2
void parser::
parse_source (token& t, type& tt)
{
+ // source [<attrs>] <path>+
+ //
+
// The rest should be a list of buildfiles. Parse them as names in the
// value mode to get variable expansion and directory prefixes.
//
mode (lexer_mode::value, '@');
- next (t, tt);
+ next_with_attributes (t, tt);
+ attributes_push (t, tt);
+
+ bool nodt (false); // Source buildfile without default target semantics.
+ {
+ attributes as (attributes_pop ());
+ const location& l (as.loc);
+
+ for (const attribute& a: as)
+ {
+ const string& n (a.name);
+
+ if (n == "no_default_target")
+ {
+ nodt = true;
+ }
+ else
+ fail (l) << "unknown source directive attribute " << a;
+ }
+ }
+
const location l (get_location (t));
names ns (tt != type::newline && tt != type::eos
? parse_names (t, tt, pattern_mode::expand, "path", nullptr)
@@ -3264,7 +3289,7 @@ namespace build2
source_buildfile (ifs,
path_name (p),
get_location (t),
- false /* default_target */);
+ nodt ? optional<bool> {} : false);
}
catch (const io_error& e)
{
@@ -3278,6 +3303,9 @@ namespace build2
void parser::
parse_include (token& t, type& tt)
{
+ // include <path>+
+ //
+
tracer trace ("parser::parse_include", &path_);
if (stage_ == stage::boot)
@@ -4072,6 +4100,7 @@ namespace build2
bool meta (false); // Import with metadata.
bool once (false); // Import buildfile once.
+ bool nodt (false); // Import buildfile without default target semantics.
{
attributes& as (attributes_top ());
const location& l (as.loc);
@@ -4089,6 +4118,10 @@ namespace build2
meta = true;
}
+ else if (n == "no_default_target")
+ {
+ nodt = true;
+ }
else if (n == "once")
{
once = true;
@@ -4213,8 +4246,15 @@ namespace build2
if (meta)
fail (loc) << "metadata requested for buildfile target " << n;
- if (once && var != nullptr)
- fail (loc) << "once importation requested with variable assignment";
+ if (var != nullptr)
+ {
+ if (once)
+ fail (loc) << "once importation requested with variable assignment";
+
+ if (nodt)
+ fail (loc) << "no_default_target importation requested with "
+ << "variable assignment";
+ }
if (ph2 && !ph2->empty ())
fail (loc) << "rule hint specified for buildfile target " << n;
@@ -4224,6 +4264,10 @@ namespace build2
if (once)
fail (loc) << "once importation requested for target " << n;
+ if (nodt)
+ fail (loc) << "no_default_target importation requested for target "
+ << n;
+
if (var == nullptr)
fail (loc) << "variable assignment required to import target " << n;
}
@@ -4323,7 +4367,7 @@ namespace build2
source_buildfile (ifs,
path_name (p),
loc,
- false /* default_target */);
+ nodt ? optional<bool> {} : false);
}
catch (const io_error& e)
{
@@ -3230,11 +3231,35 @@ namespace build2
// The rest should be a list of buildfiles. Parse them as names in the
- // value mode to get variable expansion and directory prefixes.
+ // value mode to get variable expansion and directory prefixes. Also
+ // handle optional attributes.
//
mode (lexer_mode::value, '@');